from tracardi_dot_notation.dot_accessor import DotAccessor
from tracardi_dot_notation.dot_template import DotTemplate
from tracardi_plugin_sdk.action_runner import ActionRunner
from tracardi_plugin_sdk.domain.register import Plugin, Spec, MetaData, Form, FormGroup, FormField, FormComponent
from tracardi_language_detection.model.configuration import Key, Configuration
from tracardi.service.storage.driver import storage
from tracardi_language_detection.service.sendman import PostMan


def validate(config: dict):
    return Configuration(**config)


class DetectAction(ActionRunner):

    @staticmethod
    async def build(**kwargs) -> 'DetectAction':
        # This reads config
        config = validate(kwargs)

        # This reads resource
        source = await storage.driver.resource.load(config.source.id)

        return DetectAction(config.message, Key(**source.config))

    def __init__(self, message: str, key: Key):
        self.message = message
        self.postman = PostMan(key.token)

    async def run(self, payload):
        dot = DotAccessor(self.profile, self.session, payload, self.event, self.flow)
        template = DotTemplate()
        string = template.render(self.message, dot)
        return await self.postman.send(string)


def register() -> Plugin:
    return Plugin(
        start=False,
        spec=Spec(
            module='tracardi_language_detection.plugin',
            className='DetectAction',
            inputs=["payload"],
            outputs=['payload'],
            version='0.1.2',
            license="MIT",
            author="Patryk Migaj",
            init={
                'source': {
                    'id': None
                },
                "message": "Hello world"
            },
            form=Form(groups=[
                FormGroup(
                    fields=[
                        FormField(
                            id="source",
                            name="Token resource",
                            description="Select resource that have API token.",
                            component=FormComponent(type="resource", props={"label": "resource"})
                        )
                    ]
                ),
                FormGroup(
                    fields=[
                        FormField(
                            id="message",
                            name="Text",
                            description="Type text or path to text to be detected.",
                            component=FormComponent(type="textarea", props={"label": "template"})
                        )
                    ]
                )
            ]
            )
        ),
        metadata=MetaData(
            name='Language detection',
            desc='This plugin detect language from given string with meaningcloud API',
            type='flowNode',
            width=200,
            height=100,
            icon='icon',
            group=["General"]
        )
    )



from tracardi.domain.context import Context
from tracardi.domain.entity import Entity
from tracardi.domain.event import Event
from tracardi.domain.profile import Profile
from tracardi.domain.session import Session
from tracardi_plugin_sdk.service.plugin_runner import run_plugin

from tracardi_language_detection.plugin import DetectAction

init = payload = {
    'source': {
        'id': '32ccc6e3-0084-4989-8ce2-edd35d888f29'
    },
    "message": """Welcome aboard Please pay attention as we demonstrate The safety features of 
                     this aircraft""",
    }

payload = {}
profile = Profile(id="profile-id")
event = Event(id="event-id",
              type="event-type",
              profile=profile,
              session=Session(id="session-id"),
              source=Entity(id="source-id"),
              context=Context())
result = run_plugin(DetectAction, init, payload,
                    profile, event)

print("OUTPUT:", result.output)
print("PROFILE:", result.profile)



import asyncio
import aiohttp
from aiohttp import ClientConnectorError

from tracardi_plugin_sdk.domain.result import Result


class PostMan():
    def __init__(self, key):
        self.key = key

    async def send(self, string):
        try:
            timeout = aiohttp.ClientTimeout(total=15)
            async with aiohttp.ClientSession(timeout=timeout) as session:
                params = {
                    'key': self.key,
                    'txt': string
                }
                async with session.request(
                        method="POST",
                        url=str("https://api.meaningcloud.com/lang-4.0/identification"),
                        data=params
                ) as response:
                    result = {
                        "status": response.status,
                        "body": await response.json()
                    }

                    if response.status in [200, 201, 202, 203, 204]:

                        return Result(port="response", value=result), Result(port="error", value=None)
                    else:
                        return Result(port="response", value=None), Result(port="error", value=result)
        except ClientConnectorError as e:
            return Result(port="response", value=None), Result(port="error", value=str(e))

        except asyncio.exceptions.TimeoutError:

            return Result(port="response", value=None), Result(port="error", value="Timeout.")



from pydantic import BaseModel, validator
from tracardi.domain.entity import Entity


class Key(BaseModel):
    token: str

    @validator("token")
    def must_not_be_empty(cls, value):
        if len(value) < 1:
            raise ValueError("Token must not be empty.")
        return value


class Configuration(BaseModel):
    source: Entity
    message: str

    @validator("message")
    def must_not_be_empty(cls, value):
        if len(value) < 2:
            raise ValueError("Message must not be empty.")
        return value



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='tracardi-language-detection',
    version='0.1.2',
    description='This plugin detect language from given string with meaningcloud API',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Patryk Migaj',
    author_email='patromi123@gmail.com',
    packages=['tracardi_language_detection'],
    install_requires=[
        'tracardi-plugin-sdk>=0.6.21',
        'tracardi>=0.6.18,<0.7.0'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ],
    keywords=['tracardi', 'plugin'],
    include_package_data=True,
    python_requires=">=3.8",
)